C***********************************************************************
C*
C*  Sample fortran program to write a single channel 
C*  MiniSEED data file.
C*
C*	Douglas Neuhauser						
C*	Seismographic Station						
C*	University of California, Berkeley				
C*	doug@seismo.berkeley.edu					
C*									
C***********************************************************************

C*
C* Copyright (c) 1999 The Regents of the University of California.
C* All Rights Reserved.
C* 
C* Permission to use, copy, modify, and distribute this software and its
C* documentation for educational, research and non-profit purposes,
C* without fee, and without a written agreement is hereby granted,
C* provided that the above copyright notice, this paragraph and the
C* following three paragraphs appear in all copies.
C* 
C* Permission to incorporate this software into commercial products may
C* be obtained from the Office of Technology Licensing, 2150 Shattuck
C* Avenue, Suite 510, Berkeley, CA  94704.
C* 
C* IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
C* FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
C* INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
C* ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
C* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
C* 
C* THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
C* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
C* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
C* PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
C* CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
C* UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
C*

c $Id: mswrite.f,v 1.1 1999/04/04 14:28:40 doug Exp $ 

	implicit none
	include "../qlib2/qlib2.inc"
c:: 	include "/usr/local/include/qlib2.inc"

	integer BLKSIZE, MAXBLKS
	parameter (BLKSIZE = 4096, MAXBLKS = 10)
	character mseed*(BLKSIZE*MAXBLKS)
	integer MAXPTS
	parameter (MAXPTS = 3500000)
	double precision THRESHOLD
	parameter (THRESHOLD = .5)

	record /DATA_HDR/ hdr
	record /INT_TIME/ it
	record /EXT_TIME/ et
	integer data(MAXPTS)
	integer fp, remaining, next, n, i
	integer nsamples, nrecords, nwrite, ndata
	integer nargs
	integer my_wordorder
	integer status
	character*80 file

c external functions and subroutines
	integer iargc
	external getarg, iargc
	integer lnblnk
	external lnblnk 
c functions in qlib2
	integer f_get_my_wordorder, f_mdy_to_doy
	external f_get_my_wordorder, f_mdy_to_doy
	external f_set_hdr_wordorder, f_set_data_wordorder
 	external f_ext_to_int, f_init_data_hdr
	integer f_delete_blockette
	external f_delete_blockette
c functions in library fio.
	integer ifopen, ifclose, ifwrite
	external ifopen, ifclose, ifwrite

	
c read filename from command line argument and open input miniseed file.
	nargs = iargc()
	if (nargs .lt. 1) then
	    write (*,*) "Missing filename"
	    call exit(1)
	endif
	call getarg (1, file)		
	
	fp = ifopen (file, "w")
	if (fp .eq. 0) then
		write(*,*) "unable to open file", file
		call exit(1)
	endif

c set number of requested data points.
c NOTE -- buffer must be large enough to handle this many points.
	nsamples = 50000
	do i = 1, nsamples
	    data(i) = i
	end do

c Initialize the desired wordorder for the data.
	my_wordorder = f_get_my_wordorder()
	i = f_set_hdr_wordorder(my_wordorder)
	i = f_set_data_wordorder(my_wordorder)
c
c Compute the time of the first sample.
c
	et.year = 1996
	et.month = 12
	et.day = 25
	et.hour = 8
	et.minute = 25
	et.second = 30.
	et.usec = 500000
	et.doy = f_mdy_to_doy (et.month, et.day, et.year)
	call f_ext_to_int (et, it)	
c
c Set up all of the required data in the header.
c
	call f_init_data_hdr (hdr)
	hdr.station_id = 'TEST'
	hdr.channel_id = 'CHN'
	hdr.network_id = 'XX'
	hdr.sample_rate = 20
	hdr.sample_rate_mult = 1
	hdr.data_type = STEIM1
	hdr.blksize = 4096

	hdr.hdrtime = it
        hdr.begtime = hdr.hdrtime
	hdr.seq_no = 1
	remaining = nsamples
c
c Loop over the data, packing it into MiniSEED and writing the 
c MiniSEED records to a file.  Continue until all data is processed.
c You REALLY want the mseed buffer large enough to hold your entire
c data array, since otherwise the MiniSEED compressor restarts
c every time you call it.
c
	do while (remaining .gt. 0)
	    next = nsamples - remaining + 1
	    hdr.num_samples = remaining
c
c Create the MiniSEED data.  If the output buffer is not large enough
c hold the packed MiniSEED for all of the input data, only a portion
c of the data will be packed.  The function will return the number
c of data points packed into the MiniSEED buffer.
c
	    ndata = f_ms_pack_data (hdr, hdr.num_samples, data(next), 
     1				nrecords, mseed, BLKSIZE * MAXBLKS)
c
c Write the MiniSEED data to the file.
c
	    nwrite = ifwrite (mseed, hdr.blksize * nrecords, 1, fp)
	    if (nwrite .ne. 1) then
		write (*,'(A A)') 'Error writing MiniSEED file', file
		call exit (1)
	    endif
	
c
c Update the hdr for the next chunk of data
c
	    status = f_ms_pack_update_hdr (hdr, nrecords, ndata, data(next))
	    n = f_delete_blockette (hdr, -1)
	    remaining = remaining - ndata
	    if (remaining .gt. 0) then
		write (*,'(A,I,A)' ) 'Warning - mseed buffer too small',
     1				   remaining, ' points process next pass.'
	    endif
	end do
c
c CLose the file.
c
	i = ifclose (fp)
c
	stop
	end
